# Static Exporting

There are two ways to run a Perseus app: with a server, and without a server. This might seem odd, how could you run an app without a server? Well, you can't really. But, you can *export* your app to static files and then use a stock server to just host those for you. For example, [GitHub Pages]() is a hosting service free for open-source projects that can host static files, and that's what this website runs on! There's no Perseus server behind this, just static files that you could serve with a Python script if you wanted to.

Most of Perseus' features don't actually need a server, though some do. Specifically, if your app uses incremental generation, revalidation, or request state, attempting to export your app will fail.

But, if your app doesn't use any of those, you can run `perseus export` and that will export your app to a series of static files! For convenience, `perseus export -s` will spin up a server built into the CLI so you can see what everything looks like. When you're ready to go to production, you can run `perseus deploy -e` to get a `pkg/` directory with your static files, which can be sent off to any file hosting platform!

The one big caveat with this approach is error pages. When you're using Perseus's server, it's smart enough to use your error pages when it undergoes a 404, 500, 418, etc. error, but a run-of-the-mill static file server will be clueless (this goes for the development server from `perseus export -s` as well). Usually, you'll need to explicitly provide something like `404.html` as a file to tell these servers what to provide to users. However, confusingly, your error pages will work in some circumstances. Specifically, if the user clicks a link inside your app that goes to a page that doesn't exist, this involves asking for a page from the server in the background, and the app will automatically return your error page from within itself in this case. If you go to a nonexistent page from *outside* your app though, you'll get some stock error page that's got nothing to do with Perseus. This might be hard to understand, which is why it's best to do the following.

You can solve this problem by exporting your error pages to static files with the `perseus export-error-page --code <http-code> --output <output>` command, replacing `<http-code>` with the code you want to export for (e.g. `404`) and `<output>` with where you want to put the file. These won't work at all with the development server, which isn't designed to handle error pages (yet), but a production file server should manage this fine. (Your mileage may vary depending on the hosting provider, so it's best to check first!)

*Note: apps using exporting only should see [these examples]() for how to avoid having to import a server in `Cargo.toml`.*
